#cs
	HexControl 1.0
	All rights reserved
	(c) Max Reble 2014

	HexControl UDF is licensed under a
	Creative Commons Attribution 4.0
	International License.
	http://creativecommons.org/licenses/by/4.0/
#ce


#include-once

#include <EditConstants.au3>
#include <GUIConstantsEx.au3>
#include <GuiEdit.au3>
#include <SendMessage.au3>
#include <StringConstants.au3>
#include <WinAPI.au3>
#include <WindowsConstants.au3>






; #FUNCTION# ======================================================================================
; Name ..........: _GuiCtrlHex_Create()
; Description ...: Creates the Hex Control
; Syntax ........: _GuiCtrlHex_Create($hWnd, $iX, $iY[, $iWidth = 150[, $iHeight = 150[, $fHorizontal = True[, $vWM_COMMAND = False[, $sFont = "Courier New"[, $iFontSize = 14[, $iStyle = 0x00211004[, $iExStyle = 0x00000200]]]]]]]])
; Parameters ....: $hWnd        - The handle of the GUI
;                  $iX          - Left
;                  $iY          - Right
;                  $iWidth      - [optional] Width (default:150)
;                  $iHeight     - [optional] Heigth (default:150)
;                  $fHorizontal  - [optional] If True the Controls are placed horizontally, if False vertically (default:True)
;                  $vWM_COMMAND  - [optional] If you use another WM_COMMAND function you can specify it here (As function, not as string) (default:False)
;                  $sFont       - [optional] Font Name (default:"Courier New")
;                  $iFontSize   - [optional] Font Size (default:14)
;                  $iStyle      - [optional] Style of the Edit (default:0x00211004)
;                  $iExStyle    - [optional] ExStyle of the Edit (default:0x00000200)
; Return values .: Array of the Control
; Author ........: Max Reble
; =================================================================================================
Func _GuiCtrlHex_Create($hWnd, $iX, $iY, $iWidth = 150, $iHeight = 150, $fHorizontal = True, $vWM_COMMAND = False, $sFont = "Courier New", $iFontSize = 14, $iStyle = 0x00211004, $iExStyle = 0x00000200)
	Local $aRet[3]
	Local Static $fIsRegistered = False
	Local $hFont = _WinAPI_CreateFont($iFontSize, 0, Default, Default, Default, Default, Default, Default, Default, _
			Default, Default, Default, Default, $sFont)



	If $fHorizontal Then
		$iWidth = ($iWidth / 2) - 4
		$aRet[0] = _GUICtrlEdit_Create($hWnd, "", $iX, $iY, $iWidth, $iHeight, $iStyle, $iExStyle)
		$aRet[1] = _GUICtrlEdit_Create($hWnd, "", $iX + $iWidth + 8, $iY, $iWidth, $iHeight, $iStyle, $iExStyle)
	Else
		$iHeight = ($iHeight / 2) - 4
		$aRet[0] = _GUICtrlEdit_Create($hWnd, "", $iX, $iY, $iWidth, $iHeight, $iStyle, $iExStyle)
		$aRet[1] = _GUICtrlEdit_Create($hWnd, "", $iX, $iY + $iHeight + 8, $iWidth, $iHeight, $iStyle, $iExStyle)
	EndIf

	$aRet[2] = $hWnd

	_SendMessage($aRet[0], $WM_SETFONT, $hFont, True)
	_SendMessage($aRet[1], $WM_SETFONT, $hFont, True)

	If Not $fIsRegistered Then
		GUIRegisterMsg($WM_COMMAND, "WM_COMMAND")
		$fIsRegistered = True
	EndIf

	WM_COMMAND(-1, $aRet, $vWM_COMMAND, 0)

	Return $aRet
EndFunc   ;==>_GuiCtrlHex_Create

; #FUNCTION# ======================================================================================
; Name ..........: _GuiCtrlHex_SetTextBinary()
; Description ...: Set binary data
; Syntax ........: _GuiCtrlHex_SetTextBinary($ahCtrl, $sHex])
; Parameters ....: $ahCtrl  - The Array of the controls returned by _GuiCtrlHex_Create
;                  $sHex   - The hex data in format "0123456789ABCDEF"
; Return values .: The formated hex string
; Author ........: Max Reble
; =================================================================================================
Func _GuiCtrlHex_SetTextBinary($ahCtrl, $sHex)
	If Not IsArray($ahCtrl) Then Return SetError(1, 0, 0)
	Local $iLen, $sStr

	;Upper da hex!
	$sHex = StringUpper($sHex)

	;Delete every char except the hex chars
	$sHex = StringRegExpReplace($sHex, "[^0-9A-F]", "")



	;If hex has not valid len, delete one char from the rigth for the binary conversation
	If Mod(StringLen($sHex), 2) Then
		$sStr = StringReplace(BinaryToString("0x" & StringTrimRight($sHex, 1)), Chr(0), Chr(255))
	Else
		$sStr = StringReplace(BinaryToString("0x" & $sHex), Chr(0), Chr(255))
	EndIf
	;Set the string Edit
	_GUICtrlEdit_SetText($ahCtrl[1], $sStr)

	;every 16 chars (8bits) a linefeed
	$sHex = StringRegExpReplace($sHex, ".{16}", "$0" & @CRLF);8bit
	;every 2 chars a whitespace
	$sHex = StringStripWS(StringRegExpReplace($sHex, ".{1,2}", "$0 "), $STR_STRIPTRAILING)

	;Set the Text of the hexctrl
	_GUICtrlEdit_SetText($ahCtrl[0], $sHex)

	Return $sHex
EndFunc   ;==>_GuiCtrlHex_SetTextBinary

; #FUNCTION# ======================================================================================
; Name ..........: _GuiCtrlHex_SetTextString()
; Description ...: Set a string to the controls
; Syntax ........: _GuiCtrlHex_SetTextString($ahCtrl, $sString])
; Parameters ....: $ahCtrl  - The Array of the controls returned by _GuiCtrlHex_Create
;                  $sString  - The String
; Return values .: The formated hex string
; Author ........: Max Reble
; =================================================================================================
Func _GuiCtrlHex_SetTextString($ahCtrl, $sString)
	If Not IsArray($ahCtrl) Then Return SetError(1, 0, 0)

	Return _GuiCtrlHex_SetTextBinary($ahCtrl, StringTrimLeft(Binary($sString), 2))
EndFunc   ;==>_GuiCtrlHex_SetTextString

Func WM_COMMAND($hWnd, $Msg, $wParam, $lParam)
	Local Static $avHexCtrls[1][3], $iUbound = 0, $vCallNext = False

	;Internal, so we dont have to use global variables but statics
	If $hWnd = -1 And UBound($Msg) = 3 Then
		ReDim $avHexCtrls[$iUbound + 1][3]
		$avHexCtrls[$iUbound][0] = $Msg[0]
		$avHexCtrls[$iUbound][1] = $Msg[1]
		$avHexCtrls[$iUbound][2] = $Msg[2]
		$iUbound += 1
		$vCallNext = $wParam
		Return True
	EndIf

	Local $hWndCtrl = $lParam
	Local $nEvent = BitShift($wParam, 16)
	Local $nHexCtrls

	If $nEvent = $EN_CHANGE Then
		For $nHexCtrls = 0 To $iUbound - 1
			If $hWnd = $avHexCtrls[$nHexCtrls][2] Then;doesn't need because we use the hWnd of the ctrl, and this is unique.. but.. nahh
				Local $ahCtrl[3] = [$avHexCtrls[$nHexCtrls][0], $avHexCtrls[$nHexCtrls][1], $avHexCtrls[$nHexCtrls][2]]
				Local $sGetText, $iActualLine, $aSelection
				If $hWndCtrl = $avHexCtrls[$nHexCtrls][0] Then;If is sligthly faster then switch
					$iActualLine = _GUICtrlEdit_GetFirstVisibleLine($avHexCtrls[$nHexCtrls][0])
					$aSelection = _GUICtrlEdit_GetSel($avHexCtrls[$nHexCtrls][0])
					Local $sNew

					;Get the text with upperstring
					$sGetText = _GUICtrlEdit_GetText($avHexCtrls[$nHexCtrls][0])

					;If cursor isn't at the end, replace the chars
					If $aSelection[0] < StringLen($sGetText) And Not StringRegExp($sGetText, "[^0-9A-Fa-f \r\n]") Then $sGetText = StringLeft($sGetText, $aSelection[0]) & StringMid($sGetText, $aSelection[0] + 2)

					;Set the text and receive the new string
					$sNew = _GuiCtrlHex_SetTextBinary($ahCtrl, $sGetText)

					ConsoleWrite($aSelection[0] & "<" & StringLen($sGetText) & @CRLF)
					If $aSelection[0] < StringLen($sGetText) Then
						;If the next sign is a whitespace set the cursor one unit to the rigth, if it's @CR (+@LF) than set it two units to the rigth
						If StringMid($sGetText, $aSelection[0] + 1, 1) = " " Then $aSelection[0] += 1
						If StringMid($sGetText, $aSelection[0] + 1, 1) = @CR Then $aSelection[0] += 2

						;Reset the Scroll and the cursor
						_GUICtrlEdit_SetSel($avHexCtrls[$nHexCtrls][0], $aSelection[0], $aSelection[0])
						_GUICtrlEdit_LineScroll($avHexCtrls[$nHexCtrls][0], 0, $iActualLine)
					Else
						_GUICtrlEdit_SetSel($avHexCtrls[$nHexCtrls][0], StringLen($sNew), StringLen($sNew))
					EndIf
				ElseIf $hWndCtrl = $avHexCtrls[$nHexCtrls][1] Then
					$aSelection = _GUICtrlEdit_GetSel($avHexCtrls[$nHexCtrls][1])
					$iActualLine = _GUICtrlEdit_GetFirstVisibleLine($avHexCtrls[$nHexCtrls][1])
					$sGetText = StringUpper(_GUICtrlEdit_GetText($avHexCtrls[$nHexCtrls][1]))
					_GuiCtrlHex_SetTextBinary($ahCtrl, StringTrimLeft(Binary(_GUICtrlEdit_GetText($hWndCtrl)), 2))

					;Reset the Scroll and the cursor
					_GUICtrlEdit_SetSel($avHexCtrls[$nHexCtrls][1], $aSelection[0], $aSelection[0])
					_GUICtrlEdit_LineScroll($avHexCtrls[$nHexCtrls][1], 0, $iActualLine)


				EndIf
			EndIf
		Next
	EndIf

	If $vCallNext Then $vCallNext($hWnd, $Msg, $wParam, $lParam)


	Return $GUI_RUNDEFMSG
EndFunc   ;==>WM_COMMAND
